"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputaudio import InputAudio, InputAudioTypedDict
from .inputfile import InputFile, InputFileTypedDict
from .inputimage import InputImage, InputImageTypedDict
from .inputtext import InputText, InputTextTypedDict
from .outputitemimagegenerationcall import (
    OutputItemImageGenerationCall,
    OutputItemImageGenerationCallTypedDict,
)
from .outputmessage import OutputMessage, OutputMessageTypedDict
from .toolcallstatusenum import ToolCallStatusEnum
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


BaseInputsTypeFunctionCall = Literal["function_call",]


class BaseInputsFunctionCallTypedDict(TypedDict):
    type: BaseInputsTypeFunctionCall
    call_id: str
    name: str
    arguments: str
    id: NotRequired[str]
    status: NotRequired[Nullable[ToolCallStatusEnum]]


class BaseInputsFunctionCall(BaseModel):
    type: BaseInputsTypeFunctionCall

    call_id: str

    name: str

    arguments: str

    id: Optional[str] = None

    status: Annotated[
        OptionalNullable[ToolCallStatusEnum], PlainValidator(validate_open_enum(False))
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "status"]
        nullable_fields = ["status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


BaseInputsTypeFunctionCallOutput = Literal["function_call_output",]


BaseInputsOutput1TypedDict = TypeAliasType(
    "BaseInputsOutput1TypedDict",
    Union[InputTextTypedDict, InputImageTypedDict, InputFileTypedDict],
)


BaseInputsOutput1 = Annotated[
    Union[
        Annotated[InputText, Tag("input_text")],
        Annotated[InputImage, Tag("input_image")],
        Annotated[InputFile, Tag("input_file")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


BaseInputsOutput2TypedDict = TypeAliasType(
    "BaseInputsOutput2TypedDict", Union[str, List[BaseInputsOutput1TypedDict]]
)


BaseInputsOutput2 = TypeAliasType(
    "BaseInputsOutput2", Union[str, List[BaseInputsOutput1]]
)


class BaseInputsFunctionCallOutputTypedDict(TypedDict):
    type: BaseInputsTypeFunctionCallOutput
    call_id: str
    output: BaseInputsOutput2TypedDict
    id: NotRequired[Nullable[str]]
    status: NotRequired[Nullable[ToolCallStatusEnum]]


class BaseInputsFunctionCallOutput(BaseModel):
    type: BaseInputsTypeFunctionCallOutput

    call_id: str

    output: BaseInputsOutput2

    id: OptionalNullable[str] = UNSET

    status: Annotated[
        OptionalNullable[ToolCallStatusEnum], PlainValidator(validate_open_enum(False))
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "status"]
        nullable_fields = ["id", "status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


BaseInputsTypeMessage2 = Literal["message",]


BaseInputsRoleDeveloper2 = Literal["developer",]


BaseInputsRoleSystem2 = Literal["system",]


BaseInputsRoleUser2 = Literal["user",]


BaseInputsRoleUnion2TypedDict = TypeAliasType(
    "BaseInputsRoleUnion2TypedDict",
    Union[BaseInputsRoleUser2, BaseInputsRoleSystem2, BaseInputsRoleDeveloper2],
)


BaseInputsRoleUnion2 = TypeAliasType(
    "BaseInputsRoleUnion2",
    Union[BaseInputsRoleUser2, BaseInputsRoleSystem2, BaseInputsRoleDeveloper2],
)


BaseInputsContent3TypedDict = TypeAliasType(
    "BaseInputsContent3TypedDict",
    Union[
        InputTextTypedDict, InputAudioTypedDict, InputImageTypedDict, InputFileTypedDict
    ],
)


BaseInputsContent3 = Annotated[
    Union[
        Annotated[InputText, Tag("input_text")],
        Annotated[InputImage, Tag("input_image")],
        Annotated[InputFile, Tag("input_file")],
        Annotated[InputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class BaseInputsMessage2TypedDict(TypedDict):
    id: str
    role: BaseInputsRoleUnion2TypedDict
    content: List[BaseInputsContent3TypedDict]
    type: NotRequired[BaseInputsTypeMessage2]


class BaseInputsMessage2(BaseModel):
    id: str

    role: BaseInputsRoleUnion2

    content: List[BaseInputsContent3]

    type: Optional[BaseInputsTypeMessage2] = None


BaseInputsTypeMessage1 = Literal["message",]


BaseInputsRoleDeveloper1 = Literal["developer",]


BaseInputsRoleAssistant = Literal["assistant",]


BaseInputsRoleSystem1 = Literal["system",]


BaseInputsRoleUser1 = Literal["user",]


BaseInputsRoleUnion1TypedDict = TypeAliasType(
    "BaseInputsRoleUnion1TypedDict",
    Union[
        BaseInputsRoleUser1,
        BaseInputsRoleSystem1,
        BaseInputsRoleAssistant,
        BaseInputsRoleDeveloper1,
    ],
)


BaseInputsRoleUnion1 = TypeAliasType(
    "BaseInputsRoleUnion1",
    Union[
        BaseInputsRoleUser1,
        BaseInputsRoleSystem1,
        BaseInputsRoleAssistant,
        BaseInputsRoleDeveloper1,
    ],
)


BaseInputsContent1TypedDict = TypeAliasType(
    "BaseInputsContent1TypedDict",
    Union[
        InputTextTypedDict, InputAudioTypedDict, InputImageTypedDict, InputFileTypedDict
    ],
)


BaseInputsContent1 = Annotated[
    Union[
        Annotated[InputText, Tag("input_text")],
        Annotated[InputImage, Tag("input_image")],
        Annotated[InputFile, Tag("input_file")],
        Annotated[InputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


BaseInputsContent2TypedDict = TypeAliasType(
    "BaseInputsContent2TypedDict", Union[List[BaseInputsContent1TypedDict], str]
)


BaseInputsContent2 = TypeAliasType(
    "BaseInputsContent2", Union[List[BaseInputsContent1], str]
)


BaseInputsPhaseFinalAnswer = Literal["final_answer",]


BaseInputsPhaseCommentary = Literal["commentary",]


BaseInputsPhaseUnionTypedDict = TypeAliasType(
    "BaseInputsPhaseUnionTypedDict",
    Union[BaseInputsPhaseCommentary, BaseInputsPhaseFinalAnswer, Any],
)


BaseInputsPhaseUnion = TypeAliasType(
    "BaseInputsPhaseUnion",
    Union[BaseInputsPhaseCommentary, BaseInputsPhaseFinalAnswer, Any],
)


class BaseInputsMessage1TypedDict(TypedDict):
    role: BaseInputsRoleUnion1TypedDict
    content: BaseInputsContent2TypedDict
    type: NotRequired[BaseInputsTypeMessage1]
    phase: NotRequired[Nullable[BaseInputsPhaseUnionTypedDict]]


class BaseInputsMessage1(BaseModel):
    role: BaseInputsRoleUnion1

    content: BaseInputsContent2

    type: Optional[BaseInputsTypeMessage1] = None

    phase: OptionalNullable[BaseInputsPhaseUnion] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "phase"]
        nullable_fields = ["phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


BaseInputsUnion1TypedDict = TypeAliasType(
    "BaseInputsUnion1TypedDict",
    Union[
        BaseInputsMessage1TypedDict,
        BaseInputsMessage2TypedDict,
        OutputItemImageGenerationCallTypedDict,
        BaseInputsFunctionCallOutputTypedDict,
        BaseInputsFunctionCallTypedDict,
        OutputMessageTypedDict,
    ],
)


BaseInputsUnion1 = TypeAliasType(
    "BaseInputsUnion1",
    Union[
        BaseInputsMessage1,
        BaseInputsMessage2,
        OutputItemImageGenerationCall,
        BaseInputsFunctionCallOutput,
        BaseInputsFunctionCall,
        OutputMessage,
    ],
)


BaseInputsUnionTypedDict = TypeAliasType(
    "BaseInputsUnionTypedDict", Union[str, List[BaseInputsUnion1TypedDict], Any]
)


BaseInputsUnion = TypeAliasType(
    "BaseInputsUnion", Union[str, List[BaseInputsUnion1], Any]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, Nullable, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Union
from typing_extensions import Annotated, TypedDict


CompoundFilterType = Union[
    Literal[
        "and",
        "or",
    ],
    UnrecognizedStr,
]


class CompoundFilterTypedDict(TypedDict):
    r"""A compound filter that combines multiple comparison or compound filters"""

    type: CompoundFilterType
    filters: List[Dict[str, Nullable[Any]]]


class CompoundFilter(BaseModel):
    r"""A compound filter that combines multiple comparison or compound filters"""

    type: Annotated[CompoundFilterType, PlainValidator(validate_open_enum(False))]

    filters: List[Dict[str, Nullable[Any]]]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatMessageContentItemCacheControlType = Literal["ephemeral",]


ChatMessageContentItemCacheControlTTL = Union[
    Literal[
        "5m",
        "1h",
    ],
    UnrecognizedStr,
]


class ChatMessageContentItemCacheControlTypedDict(TypedDict):
    r"""Cache control for the content part"""

    type: ChatMessageContentItemCacheControlType
    ttl: NotRequired[ChatMessageContentItemCacheControlTTL]


class ChatMessageContentItemCacheControl(BaseModel):
    r"""Cache control for the content part"""

    type: ChatMessageContentItemCacheControlType

    ttl: Annotated[
        Optional[ChatMessageContentItemCacheControlTTL],
        PlainValidator(validate_open_enum(False)),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FormatTextConfigType = Literal["text",]


class FormatTextConfigTypedDict(TypedDict):
    r"""Plain text response format"""

    type: FormatTextConfigType


class FormatTextConfig(BaseModel):
    r"""Plain text response format"""

    type: FormatTextConfigType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .preview_websearchuserlocation import (
    PreviewWebSearchUserLocation,
    PreviewWebSearchUserLocationTypedDict,
)
from .searchcontextsizeenum import SearchContextSizeEnum
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


PreviewWebSearchServerToolType = Literal["web_search_preview",]


PreviewWebSearchServerToolEngine = Union[
    Literal[
        "auto",
        "native",
        "exa",
        "firecrawl",
        "parallel",
    ],
    UnrecognizedStr,
]
r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""


class PreviewWebSearchServerToolFiltersTypedDict(TypedDict):
    allowed_domains: NotRequired[Nullable[List[str]]]
    excluded_domains: NotRequired[Nullable[List[str]]]


class PreviewWebSearchServerToolFilters(BaseModel):
    allowed_domains: OptionalNullable[List[str]] = UNSET

    excluded_domains: OptionalNullable[List[str]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["allowed_domains", "excluded_domains"]
        nullable_fields = ["allowed_domains", "excluded_domains"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PreviewWebSearchServerToolTypedDict(TypedDict):
    r"""Web search preview tool configuration"""

    type: PreviewWebSearchServerToolType
    search_context_size: NotRequired[SearchContextSizeEnum]
    r"""Size of the search context for web search tools"""
    user_location: NotRequired[Nullable[PreviewWebSearchUserLocationTypedDict]]
    engine: NotRequired[PreviewWebSearchServerToolEngine]
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""
    max_results: NotRequired[float]
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""
    filters: NotRequired[Nullable[PreviewWebSearchServerToolFiltersTypedDict]]


class PreviewWebSearchServerTool(BaseModel):
    r"""Web search preview tool configuration"""

    type: PreviewWebSearchServerToolType

    search_context_size: Annotated[
        Optional[SearchContextSizeEnum], PlainValidator(validate_open_enum(False))
    ] = None
    r"""Size of the search context for web search tools"""

    user_location: OptionalNullable[PreviewWebSearchUserLocation] = UNSET

    engine: Annotated[
        Optional[PreviewWebSearchServerToolEngine],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""

    max_results: Optional[float] = None
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""

    filters: OptionalNullable[PreviewWebSearchServerToolFilters] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "search_context_size",
            "user_location",
            "engine",
            "max_results",
            "filters",
        ]
        nullable_fields = ["user_location", "filters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatContentCacheControlType = Literal["ephemeral",]


ChatContentCacheControlTTL = Union[
    Literal[
        "5m",
        "1h",
    ],
    UnrecognizedStr,
]


class ChatContentCacheControlTypedDict(TypedDict):
    r"""Cache control for the content part"""

    type: ChatContentCacheControlType
    ttl: NotRequired[ChatContentCacheControlTTL]


class ChatContentCacheControl(BaseModel):
    r"""Cache control for the content part"""

    type: ChatContentCacheControlType

    ttl: Annotated[
        Optional[ChatContentCacheControlTTL], PlainValidator(validate_open_enum(False))
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatassistantimages import ChatAssistantImages, ChatAssistantImagesTypedDict
from .chataudiooutput import ChatAudioOutput, ChatAudioOutputTypedDict
from .chatcontentitems import ChatContentItems, ChatContentItemsTypedDict
from .chattoolcall import ChatToolCall, ChatToolCallTypedDict
from .reasoningdetailunion import ReasoningDetailUnion, ReasoningDetailUnionTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any, List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ChatAssistantMessageRole = Literal["assistant",]


ChatAssistantMessageContentTypedDict = TypeAliasType(
    "ChatAssistantMessageContentTypedDict",
    Union[str, List[ChatContentItemsTypedDict], Any],
)
r"""Assistant message content"""


ChatAssistantMessageContent = TypeAliasType(
    "ChatAssistantMessageContent", Union[str, List[ChatContentItems], Any]
)
r"""Assistant message content"""


class ChatAssistantMessageTypedDict(TypedDict):
    r"""Assistant message for requests and responses"""

    role: ChatAssistantMessageRole
    content: NotRequired[Nullable[ChatAssistantMessageContentTypedDict]]
    r"""Assistant message content"""
    name: NotRequired[str]
    r"""Optional name for the assistant"""
    tool_calls: NotRequired[List[ChatToolCallTypedDict]]
    r"""Tool calls made by the assistant"""
    refusal: NotRequired[Nullable[str]]
    r"""Refusal message if content was refused"""
    reasoning: NotRequired[Nullable[str]]
    r"""Reasoning output"""
    reasoning_details: NotRequired[List[ReasoningDetailUnionTypedDict]]
    r"""Reasoning details for extended thinking models"""
    images: NotRequired[List[ChatAssistantImagesTypedDict]]
    r"""Generated images from image generation models"""
    audio: NotRequired[ChatAudioOutputTypedDict]
    r"""Audio output data or reference"""


class ChatAssistantMessage(BaseModel):
    r"""Assistant message for requests and responses"""

    role: ChatAssistantMessageRole

    content: OptionalNullable[ChatAssistantMessageContent] = UNSET
    r"""Assistant message content"""

    name: Optional[str] = None
    r"""Optional name for the assistant"""

    tool_calls: Optional[List[ChatToolCall]] = None
    r"""Tool calls made by the assistant"""

    refusal: OptionalNullable[str] = UNSET
    r"""Refusal message if content was refused"""

    reasoning: OptionalNullable[str] = UNSET
    r"""Reasoning output"""

    reasoning_details: Optional[List[ReasoningDetailUnion]] = None
    r"""Reasoning details for extended thinking models"""

    images: Optional[List[ChatAssistantImages]] = None
    r"""Generated images from image generation models"""

    audio: Optional[ChatAudioOutput] = None
    r"""Audio output data or reference"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "content",
            "name",
            "tool_calls",
            "refusal",
            "reasoning",
            "reasoning_details",
            "images",
            "audio",
        ]
        nullable_fields = ["content", "refusal", "reasoning"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, Nullable, UNSET_SENTINEL
from openrouter.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListKeyAssignmentsGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListKeyAssignmentsGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListKeyAssignmentsRequestTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """
    offset: NotRequired[str]
    r"""Number of records to skip for pagination"""
    limit: NotRequired[str]
    r"""Maximum number of records to return (max 100)"""


class ListKeyAssignmentsRequest(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """

    offset: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of records to skip for pagination"""

    limit: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Maximum number of records to return (max 100)"""


class ListKeyAssignmentsDataTypedDict(TypedDict):
    id: str
    r"""Unique identifier for the assignment"""
    key_hash: str
    r"""Hash of the assigned API key"""
    guardrail_id: str
    r"""ID of the guardrail"""
    key_name: str
    r"""Name of the API key"""
    key_label: str
    r"""Label of the API key"""
    assigned_by: Nullable[str]
    r"""User ID of who made the assignment"""
    created_at: str
    r"""ISO 8601 timestamp of when the assignment was created"""


class ListKeyAssignmentsData(BaseModel):
    id: str
    r"""Unique identifier for the assignment"""

    key_hash: str
    r"""Hash of the assigned API key"""

    guardrail_id: str
    r"""ID of the guardrail"""

    key_name: str
    r"""Name of the API key"""

    key_label: str
    r"""Label of the API key"""

    assigned_by: Nullable[str]
    r"""User ID of who made the assignment"""

    created_at: str
    r"""ISO 8601 timestamp of when the assignment was created"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["assigned_by"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListKeyAssignmentsResponseTypedDict(TypedDict):
    r"""List of key assignments"""

    data: List[ListKeyAssignmentsDataTypedDict]
    r"""List of key assignments"""
    total_count: float
    r"""Total number of key assignments for this guardrail"""


class ListKeyAssignmentsResponse(BaseModel):
    r"""List of key assignments"""

    data: List[ListKeyAssignmentsData]
    r"""List of key assignments"""

    total_count: float
    r"""Total number of key assignments for this guardrail"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ContentPartDoneEventType = Literal["response.content_part.done",]


ContentPartDoneEventPartTypedDict = TypeAliasType(
    "ContentPartDoneEventPartTypedDict",
    Union[
        ReasoningTextContentTypedDict,
        OpenAIResponsesRefusalContentTypedDict,
        ResponseOutputTextTypedDict,
    ],
)


ContentPartDoneEventPart = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[ReasoningTextContent, Tag("reasoning_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class ContentPartDoneEventTypedDict(TypedDict):
    r"""Event emitted when a content part is complete"""

    type: ContentPartDoneEventType
    output_index: float
    item_id: str
    content_index: float
    part: ContentPartDoneEventPartTypedDict
    sequence_number: float


class ContentPartDoneEvent(BaseModel):
    r"""Event emitted when a content part is complete"""

    type: ContentPartDoneEventType

    output_index: float

    item_id: str

    content_index: float

    part: ContentPartDoneEventPart

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatgenerationtokenusage import (
    ChatGenerationTokenUsage,
    ChatGenerationTokenUsageTypedDict,
)
from .chatstreamingchoice import ChatStreamingChoice, ChatStreamingChoiceTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatStreamingResponseChunkObject = Literal["chat.completion.chunk",]


class ErrorTypedDict(TypedDict):
    r"""Error information"""

    message: str
    r"""Error message"""
    code: float
    r"""Error code"""


class Error(BaseModel):
    r"""Error information"""

    message: str
    r"""Error message"""

    code: float
    r"""Error code"""


class ChatStreamingResponseChunkTypedDict(TypedDict):
    r"""Streaming chat completion chunk"""

    id: str
    r"""Unique chunk identifier"""
    choices: List[ChatStreamingChoiceTypedDict]
    r"""List of streaming chunk choices"""
    created: float
    r"""Unix timestamp of creation"""
    model: str
    r"""Model used for completion"""
    object: ChatStreamingResponseChunkObject
    system_fingerprint: NotRequired[str]
    r"""System fingerprint"""
    error: NotRequired[ErrorTypedDict]
    r"""Error information"""
    usage: NotRequired[ChatGenerationTokenUsageTypedDict]
    r"""Token usage statistics"""


class ChatStreamingResponseChunk(BaseModel):
    r"""Streaming chat completion chunk"""

    id: str
    r"""Unique chunk identifier"""

    choices: List[ChatStreamingChoice]
    r"""List of streaming chunk choices"""

    created: float
    r"""Unix timestamp of creation"""

    model: str
    r"""Model used for completion"""

    object: ChatStreamingResponseChunkObject

    system_fingerprint: Optional[str] = None
    r"""System fingerprint"""

    error: Optional[Error] = None
    r"""Error information"""

    usage: Optional[ChatGenerationTokenUsage] = None
    r"""Token usage statistics"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from .developermessage import DeveloperMessage, DeveloperMessageTypedDict
from .systemmessage import SystemMessage, SystemMessageTypedDict
from .toolresponsemessage import ToolResponseMessage, ToolResponseMessageTypedDict
from .usermessage import UserMessage, UserMessageTypedDict
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


MessageTypedDict = TypeAliasType(
    "MessageTypedDict",
    Union[
        SystemMessageTypedDict,
        UserMessageTypedDict,
        DeveloperMessageTypedDict,
        ToolResponseMessageTypedDict,
        AssistantMessageTypedDict,
    ],
)
r"""Chat completion message with role-based discrimination"""


Message = Annotated[
    Union[
        Annotated[SystemMessage, Tag("system")],
        Annotated[UserMessage, Tag("user")],
        Annotated[DeveloperMessage, Tag("developer")],
        Annotated[AssistantMessage, Tag("assistant")],
        Annotated[ToolResponseMessage, Tag("tool")],
    ],
    Discriminator(lambda m: get_discriminator(m, "role", "role")),
]
r"""Chat completion message with role-based discrimination"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatFormatPythonConfigType = Literal["python",]


class ChatFormatPythonConfigTypedDict(TypedDict):
    r"""Python code response format"""

    type: ChatFormatPythonConfigType


class ChatFormatPythonConfig(BaseModel):
    r"""Python code response format"""

    type: ChatFormatPythonConfigType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


ContextCompressionEngine = Literal["middle-out",]
r"""The compression engine to use. Defaults to \"middle-out\"."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OutputMessageItemRole = Literal["assistant",]


OutputMessageItemType = Literal["message",]


OutputMessageItemStatusInProgress = Literal["in_progress",]


OutputMessageItemStatusIncomplete = Literal["incomplete",]


OutputMessageItemStatusCompleted = Literal["completed",]


OutputMessageItemStatusUnionTypedDict = TypeAliasType(
    "OutputMessageItemStatusUnionTypedDict",
    Union[
        OutputMessageItemStatusCompleted,
        OutputMessageItemStatusIncomplete,
        OutputMessageItemStatusInProgress,
    ],
)


OutputMessageItemStatusUnion = TypeAliasType(
    "OutputMessageItemStatusUnion",
    Union[
        OutputMessageItemStatusCompleted,
        OutputMessageItemStatusIncomplete,
        OutputMessageItemStatusInProgress,
    ],
)


OutputMessageItemContentTypedDict = TypeAliasType(
    "OutputMessageItemContentTypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


OutputMessageItemContent = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OutputMessageItemPhaseFinalAnswer = Literal["final_answer",]


OutputMessageItemPhaseCommentary = Literal["commentary",]


OutputMessageItemPhaseUnionTypedDict = TypeAliasType(
    "OutputMessageItemPhaseUnionTypedDict",
    Union[OutputMessageItemPhaseCommentary, OutputMessageItemPhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


OutputMessageItemPhaseUnion = TypeAliasType(
    "OutputMessageItemPhaseUnion",
    Union[OutputMessageItemPhaseCommentary, OutputMessageItemPhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OutputMessageItemTypedDict(TypedDict):
    r"""An output message item"""

    id: str
    role: OutputMessageItemRole
    type: OutputMessageItemType
    content: List[OutputMessageItemContentTypedDict]
    status: NotRequired[OutputMessageItemStatusUnionTypedDict]
    phase: NotRequired[Nullable[OutputMessageItemPhaseUnionTypedDict]]
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OutputMessageItem(BaseModel):
    r"""An output message item"""

    id: str

    role: OutputMessageItemRole

    type: OutputMessageItemType

    content: List[OutputMessageItemContent]

    status: Optional[OutputMessageItemStatusUnion] = None

    phase: OptionalNullable[OutputMessageItemPhaseUnion] = UNSET
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["status", "phase"]
        nullable_fields = ["phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatWebSearchShorthandType = Union[
    Literal[
        "web_search",
        "web_search_preview",
        "web_search_preview_2025_03_11",
        "web_search_2025_08_26",
    ],
    UnrecognizedStr,
]


ChatWebSearchShorthandEngine = Union[
    Literal[
        "auto",
        "native",
        "exa",
        "firecrawl",
        "parallel",
    ],
    UnrecognizedStr,
]
r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""


ChatWebSearchShorthandSearchContextSize = Union[
    Literal[
        "low",
        "medium",
        "high",
    ],
    UnrecognizedStr,
]
r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""


ChatWebSearchShorthandTypeApproximate = Literal["approximate",]


class ChatWebSearchShorthandUserLocationTypedDict(TypedDict):
    r"""Approximate user location for location-biased results."""

    type: NotRequired[ChatWebSearchShorthandTypeApproximate]
    city: NotRequired[str]
    region: NotRequired[str]
    country: NotRequired[str]
    timezone: NotRequired[str]


class ChatWebSearchShorthandUserLocation(BaseModel):
    r"""Approximate user location for location-biased results."""

    type: Optional[ChatWebSearchShorthandTypeApproximate] = None

    city: Optional[str] = None

    region: Optional[str] = None

    country: Optional[str] = None

    timezone: Optional[str] = None


ChatWebSearchShorthandParametersEngine = Union[
    Literal[
        "auto",
        "native",
        "exa",
        "firecrawl",
        "parallel",
    ],
    UnrecognizedStr,
]
r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""


ChatWebSearchShorthandParametersSearchContextSize = Union[
    Literal[
        "low",
        "medium",
        "high",
    ],
    UnrecognizedStr,
]
r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""


ChatWebSearchShorthandParametersType = Literal["approximate",]


class ChatWebSearchShorthandParametersUserLocationTypedDict(TypedDict):
    r"""Approximate user location for location-biased results."""

    type: NotRequired[ChatWebSearchShorthandParametersType]
    city: NotRequired[str]
    region: NotRequired[str]
    country: NotRequired[str]
    timezone: NotRequired[str]


class ChatWebSearchShorthandParametersUserLocation(BaseModel):
    r"""Approximate user location for location-biased results."""

    type: Optional[ChatWebSearchShorthandParametersType] = None

    city: Optional[str] = None

    region: Optional[str] = None

    country: Optional[str] = None

    timezone: Optional[str] = None


class ChatWebSearchShorthandParametersTypedDict(TypedDict):
    engine: NotRequired[ChatWebSearchShorthandParametersEngine]
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""
    max_results: NotRequired[float]
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""
    max_total_results: NotRequired[float]
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results. Useful for controlling cost and context size in agentic loops."""
    search_context_size: NotRequired[ChatWebSearchShorthandParametersSearchContextSize]
    r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""
    user_location: NotRequired[ChatWebSearchShorthandParametersUserLocationTypedDict]
    r"""Approximate user location for location-biased results."""
    allowed_domains: NotRequired[List[str]]
    r"""Limit search results to these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""
    excluded_domains: NotRequired[List[str]]
    r"""Exclude search results from these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""


class ChatWebSearchShorthandParameters(BaseModel):
    engine: Annotated[
        Optional[ChatWebSearchShorthandParametersEngine],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""

    max_results: Optional[float] = None
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""

    max_total_results: Optional[float] = None
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results. Useful for controlling cost and context size in agentic loops."""

    search_context_size: Annotated[
        Optional[ChatWebSearchShorthandParametersSearchContextSize],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""

    user_location: Optional[ChatWebSearchShorthandParametersUserLocation] = None
    r"""Approximate user location for location-biased results."""

    allowed_domains: Optional[List[str]] = None
    r"""Limit search results to these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""

    excluded_domains: Optional[List[str]] = None
    r"""Exclude search results from these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""


class ChatWebSearchShorthandTypedDict(TypedDict):
    r"""Web search tool using OpenAI Responses API syntax. Automatically converted to openrouter:web_search."""

    type: ChatWebSearchShorthandType
    engine: NotRequired[ChatWebSearchShorthandEngine]
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""
    max_results: NotRequired[float]
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""
    max_total_results: NotRequired[float]
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results. Useful for controlling cost and context size in agentic loops."""
    search_context_size: NotRequired[ChatWebSearchShorthandSearchContextSize]
    r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""
    user_location: NotRequired[ChatWebSearchShorthandUserLocationTypedDict]
    r"""Approximate user location for location-biased results."""
    allowed_domains: NotRequired[List[str]]
    r"""Limit search results to these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""
    excluded_domains: NotRequired[List[str]]
    r"""Exclude search results from these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""
    parameters: NotRequired[ChatWebSearchShorthandParametersTypedDict]


class ChatWebSearchShorthand(BaseModel):
    r"""Web search tool using OpenAI Responses API syntax. Automatically converted to openrouter:web_search."""

    type: Annotated[
        ChatWebSearchShorthandType, PlainValidator(validate_open_enum(False))
    ]

    engine: Annotated[
        Optional[ChatWebSearchShorthandEngine],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""Which search engine to use. \"auto\" (default) uses native if the provider supports it, otherwise Exa. \"native\" forces the provider's built-in search. \"exa\" forces the Exa search API. \"firecrawl\" uses Firecrawl (requires BYOK). \"parallel\" uses the Parallel search API."""

    max_results: Optional[float] = None
    r"""Maximum number of search results to return per search call. Defaults to 5. Applies to Exa, Firecrawl, and Parallel engines; ignored with native provider search."""

    max_total_results: Optional[float] = None
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results. Useful for controlling cost and context size in agentic loops."""

    search_context_size: Annotated[
        Optional[ChatWebSearchShorthandSearchContextSize],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""How much context to retrieve per result. Defaults to medium (15000 chars). Only applies when using the Exa engine; ignored with native provider search."""

    user_location: Optional[ChatWebSearchShorthandUserLocation] = None
    r"""Approximate user location for location-biased results."""

    allowed_domains: Optional[List[str]] = None
    r"""Limit search results to these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""

    excluded_domains: Optional[List[str]] = None
    r"""Exclude search results from these domains. Applies to Exa and Parallel engines. Not supported with Firecrawl or native provider search."""

    parameters: Optional[ChatWebSearchShorthandParameters] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


ProviderSort = Union[
    Literal[
        "price",
        "throughput",
        "latency",
        "exacto",
    ],
    UnrecognizedStr,
]
r"""The provider sorting strategy (price, throughput, latency)"""



"""Auto-generated by @openrouter/cli types. Do not edit manually."""
# Run `npx @openrouter/cli types` to regenerate
# Generated: 2025-12-23T14:51:29.122Z
# Model count: 351

from typing import Literal, Union
from typing_extensions import TypeAlias

# Union type of all available model IDs on OpenRouter.
# Includes base models and their variants (e.g., ":free", ":nitro").
ModelId: TypeAlias = Literal[
    'ai21/jamba-large-1.7',
    'ai21/jamba-mini-1.7',
    'aion-labs/aion-1.0',
    'aion-labs/aion-1.0-mini',
    'aion-labs/aion-rp-llama-3.1-8b',
    'alfredpros/codellama-7b-instruct-solidity',
    'alibaba/tongyi-deepresearch-30b-a3b',
    'alibaba/tongyi-deepresearch-30b-a3b:free',
    'allenai/olmo-2-0325-32b-instruct',
    'allenai/olmo-3-32b-think:free',
    'allenai/olmo-3-7b-instruct',
    'allenai/olmo-3-7b-think',
    'allenai/olmo-3.1-32b-think:free',
    'alpindale/goliath-120b',
    'amazon/nova-2-lite-v1',
    'amazon/nova-lite-v1',
    'amazon/nova-micro-v1',
    'amazon/nova-premier-v1',
    'amazon/nova-pro-v1',
    'anthracite-org/magnum-v4-72b',
    'anthropic/claude-3-haiku',
    'anthropic/claude-3-opus',
    'anthropic/claude-3.5-haiku',
    'anthropic/claude-3.5-haiku-20241022',
    'anthropic/claude-3.5-sonnet',
    'anthropic/claude-3.7-sonnet',
    'anthropic/claude-3.7-sonnet:thinking',
    'anthropic/claude-haiku-4.5',
    'anthropic/claude-opus-4',
    'anthropic/claude-opus-4.1',
    'anthropic/claude-opus-4.5',
    'anthropic/claude-sonnet-4',
    'anthropic/claude-sonnet-4.5',
    'arcee-ai/coder-large',
    'arcee-ai/maestro-reasoning',
    'arcee-ai/spotlight',
    'arcee-ai/trinity-mini',
    'arcee-ai/trinity-mini:free',
    'arcee-ai/virtuoso-large',
    'arliai/qwq-32b-arliai-rpr-v1',
    'baidu/ernie-4.5-21b-a3b',
    'baidu/ernie-4.5-21b-a3b-thinking',
    'baidu/ernie-4.5-300b-a47b',
    'baidu/ernie-4.5-vl-28b-a3b',
    'baidu/ernie-4.5-vl-424b-a47b',
    'bytedance/ui-tars-1.5-7b',
    'cognitivecomputations/dolphin-mistral-24b-venice-edition:free',
    'cohere/command-a',
    'cohere/command-r-08-2024',
    'cohere/command-r-plus-08-2024',
    'cohere/command-r7b-12-2024',
    'deepcogito/cogito-v2-preview-llama-109b-moe',
    'deepcogito/cogito-v2-preview-llama-405b',
    'deepcogito/cogito-v2-preview-llama-70b',
    'deepcogito/cogito-v2.1-671b',
    'deepseek/deepseek-chat',
    'deepseek/deepseek-chat-v3-0324',
    'deepseek/deepseek-chat-v3.1',
    'deepseek/deepseek-prover-v2',
    'deepseek/deepseek-r1',
    'deepseek/deepseek-r1-0528',
    'deepseek/deepseek-r1-0528-qwen3-8b',
    'deepseek/deepseek-r1-0528:free',
    'deepseek/deepseek-r1-distill-llama-70b',
    'deepseek/deepseek-r1-distill-qwen-14b',
    'deepseek/deepseek-r1-distill-qwen-32b',
    'deepseek/deepseek-v3.1-terminus',
    'deepseek/deepseek-v3.1-terminus:exacto',
    'deepseek/deepseek-v3.2',
    'deepseek/deepseek-v3.2-exp',
    'deepseek/deepseek-v3.2-speciale',
    'eleutherai/llemma_7b',
    'essentialai/rnj-1-instruct',
    'google/gemini-2.0-flash-001',
    'google/gemini-2.0-flash-exp:free',
    'google/gemini-2.0-flash-lite-001',
    'google/gemini-2.5-flash',
    'google/gemini-2.5-flash-image',
    'google/gemini-2.5-flash-image-preview',
    'google/gemini-2.5-flash-lite',
    'google/gemini-2.5-flash-lite-preview-09-2025',
    'google/gemini-2.5-flash-preview-09-2025',
    'google/gemini-2.5-pro',
    'google/gemini-2.5-pro-preview',
    'google/gemini-2.5-pro-preview-05-06',
    'google/gemini-3-flash-preview',
    'google/gemini-3-pro-image-preview',
    'google/gemini-3-pro-preview',
    'google/gemma-2-27b-it',
    'google/gemma-2-9b-it',
    'google/gemma-3-12b-it',
    'google/gemma-3-12b-it:free',
    'google/gemma-3-27b-it',
    'google/gemma-3-27b-it:free',
    'google/gemma-3-4b-it',
    'google/gemma-3-4b-it:free',
    'google/gemma-3n-e2b-it:free',
    'google/gemma-3n-e4b-it',
    'google/gemma-3n-e4b-it:free',
    'gryphe/mythomax-l2-13b',
    'ibm-granite/granite-4.0-h-micro',
    'inception/mercury',
    'inception/mercury-coder',
    'inflection/inflection-3-pi',
    'inflection/inflection-3-productivity',
    'kwaipilot/kat-coder-pro:free',
    'liquid/lfm-2.2-6b',
    'liquid/lfm2-8b-a1b',
    'mancer/weaver',
    'meituan/longcat-flash-chat',
    'meta-llama/llama-3-70b-instruct',
    'meta-llama/llama-3-8b-instruct',
    'meta-llama/llama-3.1-405b',
    'meta-llama/llama-3.1-405b-instruct',
    'meta-llama/llama-3.1-405b-instruct:free',
    'meta-llama/llama-3.1-70b-instruct',
    'meta-llama/llama-3.1-8b-instruct',
    'meta-llama/llama-3.2-11b-vision-instruct',
    'meta-llama/llama-3.2-1b-instruct',
    'meta-llama/llama-3.2-3b-instruct',
    'meta-llama/llama-3.2-3b-instruct:free',
    'meta-llama/llama-3.2-90b-vision-instruct',
    'meta-llama/llama-3.3-70b-instruct',
    'meta-llama/llama-3.3-70b-instruct:free',
    'meta-llama/llama-4-maverick',
    'meta-llama/llama-4-scout',
    'meta-llama/llama-guard-2-8b',
    'meta-llama/llama-guard-3-8b',
    'meta-llama/llama-guard-4-12b',
    'microsoft/phi-3-medium-128k-instruct',
    'microsoft/phi-3-mini-128k-instruct',
    'microsoft/phi-3.5-mini-128k-instruct',
    'microsoft/phi-4',
    'microsoft/phi-4-multimodal-instruct',
    'microsoft/phi-4-reasoning-plus',
    'microsoft/wizardlm-2-8x22b',
    'minimax/minimax-01',
    'minimax/minimax-m1',
    'minimax/minimax-m2',
    'minimax/minimax-m2.1',
    'mistralai/codestral-2508',
    'mistralai/devstral-2512',
    'mistralai/devstral-2512:free',
    'mistralai/devstral-medium',
    'mistralai/devstral-small',
    'mistralai/devstral-small-2505',
    'mistralai/ministral-14b-2512',
    'mistralai/ministral-3b',
    'mistralai/ministral-3b-2512',
    'mistralai/ministral-8b',
    'mistralai/ministral-8b-2512',
    'mistralai/mistral-7b-instruct',
    'mistralai/mistral-7b-instruct-v0.1',
    'mistralai/mistral-7b-instruct-v0.2',
    'mistralai/mistral-7b-instruct-v0.3',
    'mistralai/mistral-7b-instruct:free',
    'mistralai/mistral-large',
    'mistralai/mistral-large-2407',
    'mistralai/mistral-large-2411',
    'mistralai/mistral-large-2512',
    'mistralai/mistral-medium-3',
    'mistralai/mistral-medium-3.1',
    'mistralai/mistral-nemo',
    'mistralai/mistral-saba',
    'mistralai/mistral-small-24b-instruct-2501',
    'mistralai/mistral-small-3.1-24b-instruct',
    'mistralai/mistral-small-3.1-24b-instruct:free',
    'mistralai/mistral-small-3.2-24b-instruct',
    'mistralai/mistral-small-creative',
    'mistralai/mistral-tiny',
    'mistralai/mixtral-8x22b-instruct',
    'mistralai/mixtral-8x7b-instruct',
    'mistralai/pixtral-12b',
    'mistralai/pixtral-large-2411',
    'mistralai/voxtral-small-24b-2507',
    'moonshotai/kimi-dev-72b',
    'moonshotai/kimi-k2',
    'moonshotai/kimi-k2-0905',
    'moonshotai/kimi-k2-0905:exacto',
    'moonshotai/kimi-k2-thinking',
    'moonshotai/kimi-k2:free',
    'morph/morph-v3-fast',
    'morph/morph-v3-large',
    'neversleep/llama-3.1-lumimaid-8b',
    'neversleep/noromaid-20b',
    'nex-agi/deepseek-v3.1-nex-n1:free',
    'nousresearch/deephermes-3-mistral-24b-preview',
    'nousresearch/hermes-2-pro-llama-3-8b',
    'nousresearch/hermes-3-llama-3.1-405b',
    'nousresearch/hermes-3-llama-3.1-405b:free',
    'nousresearch/hermes-3-llama-3.1-70b',
    'nousresearch/hermes-4-405b',
    'nousresearch/hermes-4-70b',
    'nvidia/llama-3.1-nemotron-70b-instruct',
    'nvidia/llama-3.1-nemotron-ultra-253b-v1',
    'nvidia/llama-3.3-nemotron-super-49b-v1.5',
    'nvidia/nemotron-3-nano-30b-a3b',
    'nvidia/nemotron-3-nano-30b-a3b:free',
    'nvidia/nemotron-nano-12b-v2-vl',
    'nvidia/nemotron-nano-12b-v2-vl:free',
    'nvidia/nemotron-nano-9b-v2',
    'nvidia/nemotron-nano-9b-v2:free',
    'openai/chatgpt-4o-latest',
    'openai/codex-mini',
    'openai/gpt-3.5-turbo',
    'openai/gpt-3.5-turbo-0613',
    'openai/gpt-3.5-turbo-16k',
    'openai/gpt-3.5-turbo-instruct',
    'openai/gpt-4',
    'openai/gpt-4-0314',
    'openai/gpt-4-1106-preview',
    'openai/gpt-4-turbo',
    'openai/gpt-4-turbo-preview',
    'openai/gpt-4.1',
    'openai/gpt-4.1-mini',
    'openai/gpt-4.1-nano',
    'openai/gpt-4o',
    'openai/gpt-4o-2024-05-13',
    'openai/gpt-4o-2024-08-06',
    'openai/gpt-4o-2024-11-20',
    'openai/gpt-4o-audio-preview',
    'openai/gpt-4o-mini',
    'openai/gpt-4o-mini-2024-07-18',
    'openai/gpt-4o-mini-search-preview',
    'openai/gpt-4o-search-preview',
    'openai/gpt-4o:extended',
    'openai/gpt-5',
    'openai/gpt-5-chat',
    'openai/gpt-5-codex',
    'openai/gpt-5-image',
    'openai/gpt-5-image-mini',
    'openai/gpt-5-mini',
    'openai/gpt-5-nano',
    'openai/gpt-5-pro',
    'openai/gpt-5.1',
    'openai/gpt-5.1-chat',
    'openai/gpt-5.1-codex',
    'openai/gpt-5.1-codex-max',
    'openai/gpt-5.1-codex-mini',
    'openai/gpt-5.2',
    'openai/gpt-5.2-chat',
    'openai/gpt-5.2-pro',
    'openai/gpt-oss-120b',
    'openai/gpt-oss-120b:exacto',
    'openai/gpt-oss-120b:free',
    'openai/gpt-oss-20b',
    'openai/gpt-oss-20b:free',
    'openai/gpt-oss-safeguard-20b',
    'openai/o1',
    'openai/o1-pro',
    'openai/o3',
    'openai/o3-deep-research',
    'openai/o3-mini',
    'openai/o3-mini-high',
    'openai/o3-pro',
    'openai/o4-mini',
    'openai/o4-mini-deep-research',
    'openai/o4-mini-high',
    'opengvlab/internvl3-78b',
    'openrouter/auto',
    'openrouter/bodybuilder',
    'perplexity/sonar',
    'perplexity/sonar-deep-research',
    'perplexity/sonar-pro',
    'perplexity/sonar-pro-search',
    'perplexity/sonar-reasoning',
    'perplexity/sonar-reasoning-pro',
    'prime-intellect/intellect-3',
    'qwen/qwen-2.5-72b-instruct',
    'qwen/qwen-2.5-7b-instruct',
    'qwen/qwen-2.5-coder-32b-instruct',
    'qwen/qwen-2.5-vl-7b-instruct',
    'qwen/qwen-2.5-vl-7b-instruct:free',
    'qwen/qwen-max',
    'qwen/qwen-plus',
    'qwen/qwen-plus-2025-07-28',
    'qwen/qwen-plus-2025-07-28:thinking',
    'qwen/qwen-turbo',
    'qwen/qwen-vl-max',
    'qwen/qwen-vl-plus',
    'qwen/qwen2.5-coder-7b-instruct',
    'qwen/qwen2.5-vl-32b-instruct',
    'qwen/qwen2.5-vl-72b-instruct',
    'qwen/qwen3-14b',
    'qwen/qwen3-235b-a22b',
    'qwen/qwen3-235b-a22b-2507',
    'qwen/qwen3-235b-a22b-thinking-2507',
    'qwen/qwen3-30b-a3b',
    'qwen/qwen3-30b-a3b-instruct-2507',
    'qwen/qwen3-30b-a3b-thinking-2507',
    'qwen/qwen3-32b',
    'qwen/qwen3-4b:free',
    'qwen/qwen3-8b',
    'qwen/qwen3-coder',
    'qwen/qwen3-coder-30b-a3b-instruct',
    'qwen/qwen3-coder-flash',
    'qwen/qwen3-coder-plus',
    'qwen/qwen3-coder:exacto',
    'qwen/qwen3-coder:free',
    'qwen/qwen3-max',
    'qwen/qwen3-next-80b-a3b-instruct',
    'qwen/qwen3-next-80b-a3b-thinking',
    'qwen/qwen3-vl-235b-a22b-instruct',
    'qwen/qwen3-vl-235b-a22b-thinking',
    'qwen/qwen3-vl-30b-a3b-instruct',
    'qwen/qwen3-vl-30b-a3b-thinking',
    'qwen/qwen3-vl-32b-instruct',
    'qwen/qwen3-vl-8b-instruct',
    'qwen/qwen3-vl-8b-thinking',
    'qwen/qwq-32b',
    'raifle/sorcererlm-8x22b',
    'relace/relace-apply-3',
    'relace/relace-search',
    'sao10k/l3-euryale-70b',
    'sao10k/l3-lunaris-8b',
    'sao10k/l3.1-70b-hanami-x1',
    'sao10k/l3.1-euryale-70b',
    'sao10k/l3.3-euryale-70b',
    'stepfun-ai/step3',
    'switchpoint/router',
    'tencent/hunyuan-a13b-instruct',
    'thedrummer/cydonia-24b-v4.1',
    'thedrummer/rocinante-12b',
    'thedrummer/skyfall-36b-v2',
    'thedrummer/unslopnemo-12b',
    'thudm/glm-4.1v-9b-thinking',
    'tngtech/deepseek-r1t-chimera',
    'tngtech/deepseek-r1t-chimera:free',
    'tngtech/deepseek-r1t2-chimera',
    'tngtech/deepseek-r1t2-chimera:free',
    'tngtech/tng-r1t-chimera',
    'tngtech/tng-r1t-chimera:free',
    'undi95/remm-slerp-l2-13b',
    'x-ai/grok-3',
    'x-ai/grok-3-beta',
    'x-ai/grok-3-mini',
    'x-ai/grok-3-mini-beta',
    'x-ai/grok-4',
    'x-ai/grok-4-fast',
    'x-ai/grok-4.1-fast',
    'x-ai/grok-code-fast-1',
    'xiaomi/mimo-v2-flash:free',
    'z-ai/glm-4-32b',
    'z-ai/glm-4.5',
    'z-ai/glm-4.5-air',
    'z-ai/glm-4.5-air:free',
    'z-ai/glm-4.5v',
    'z-ai/glm-4.6',
    'z-ai/glm-4.6:exacto',
    'z-ai/glm-4.6v',
    'z-ai/glm-4.7'
]

# Static variants that create distinct model IDs.
# These are already present in the API response (e.g., "openai/gpt-4:free").
StaticVariant: TypeAlias = Literal['free', 'extended', 'exacto', 'thinking']

# Virtual variants used for routing hints.
# These do not create distinct model IDs but affect routing behavior.
VirtualVariant: TypeAlias = Literal['all', 'online', 'nitro', 'floor']

# All available variants.
Variant: TypeAlias = Union[StaticVariant, VirtualVariant]

# Hash of model IDs for staleness checking.
# Used by postinstall script to detect outdated types.
MODEL_HASH: str = '986e77d89072f654'



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
import pydantic
from typing import Literal
from typing_extensions import Annotated, TypedDict


ChatMessageContentItemAudioType = Literal["input_audio",]


class ChatMessageContentItemAudioInputAudioTypedDict(TypedDict):
    data: str
    r"""Base64 encoded audio data"""
    format_: str
    r"""Audio format (e.g., wav, mp3, flac, m4a, ogg, aiff, aac, pcm16, pcm24). Supported formats vary by provider."""


class ChatMessageContentItemAudioInputAudio(BaseModel):
    data: str
    r"""Base64 encoded audio data"""

    format_: Annotated[str, pydantic.Field(alias="format")]
    r"""Audio format (e.g., wav, mp3, flac, m4a, ogg, aiff, aac, pcm16, pcm24). Supported formats vary by provider."""


class ChatMessageContentItemAudioTypedDict(TypedDict):
    r"""Audio input content part. Supported audio formats vary by provider."""

    type: ChatMessageContentItemAudioType
    input_audio: ChatMessageContentItemAudioInputAudioTypedDict


class ChatMessageContentItemAudio(BaseModel):
    r"""Audio input content part. Supported audio formats vary by provider."""

    type: ChatMessageContentItemAudioType

    input_audio: ChatMessageContentItemAudioInputAudio
